﻿
using System;
using System.Drawing;
using System.Windows.Forms;

/* Interpolation by Newton Gregory forward algorithm*/
/*        www.mosismath.com                         */

namespace WindowsFormsApplication1
{

    public partial class MainWin : Form
    {

        public const int order = 6; // number of reference points
        public const int datapoints = 1000; // number of datapoints for the interpolation
        public const double deltaX = 0.1; // sample time 

        double[] y_k = new double[order];
        double[] x_k = new double[order];
        double[] b = new double[order];
        double[] yp = new double[datapoints];
        double[] xp = new double[datapoints];
        double[] c = new double[order];


        public double Binom(double n, double k)
        {
            double i, j;
            double res = 1.0;
            j = 0;
            if ((k != 0) && (k != n))
            {
                for (i = 0; i < k; i++)
                {
                    res = res * (n - j);
                    j++;
                }
                for (i = 2.0; i <= k; i++)
                {
                    res = res / i;
                }
            }
            return res;
        }

        void CalcElements(double[] y)
        {
            int i, j;
            double sign = 1;
            for (i = 0; i < y.Length; i++)
            {
                c[i] = 0;
                sign = 1;
                for (j = 0; j < i + 1; j++)
                {
                    c[i] = c[i] + sign * y[j] * Binom(i, j);
                    sign = -sign;
                }
                if (i % 2 != 0)
                    c[i] = -c[i];
            }
        }

        double Interpolate(double xp, int order)
        {
            int i;
            double tempYp = 0;
            for (i = 0; i < order; i++)
            {
                tempYp = tempYp + Binom(xp / deltaX, i) * c[i];
            }
            return tempYp;
        }


        public MainWin()
        {
            int i;
            DataGridViewCell cell;

            InitializeComponent();

            GDataPoints.ColumnCount = order;
            GDataPoints.RowCount = 2;

            // init some sample y values
            y_k[0] = 0.5;
            y_k[1] = 11.7;
            y_k[2] = 14.8;
            y_k[3] = 4.0;
            y_k[4] = 2.2;
            y_k[5] = 0.2;

            // timestamp to y value
            for (i = 0; i < order; i++)
            {
                x_k[i] = deltaX * (double)i;
            }
            for (i = 0; i < order; i++)
            {
                cell = GDataPoints[i, 0];
                cell.Value = Convert.ToString(y_k[i]);
                cell = GDataPoints[i, 1];
                cell.Value = Convert.ToString(x_k[i]);
            }
        }


        public void DrawGraph(int maxPoints, double minX, double maxX, bool doClear)
        {
            Point p1, p2;
            int j;
            int spacing = 20;
            double maxTime = -1.0;
            double maxVal = -1.0;
            double minVal = 1.0;
            double scalefactor;
            bool drawCeroline;
            for (j = 0; j < maxPoints - 1; j++)
            {
                if ((xp[j] >= minX) && (xp[j] <= maxX))
                {
                    if (maxTime < xp[j])
                        maxTime = xp[j];
                    if (maxVal < yp[j])
                        maxVal = yp[j];
                    if (minVal > yp[j])
                        minVal = yp[j];
                }
            }
            maxTime = maxTime * 1.05;
            if (minVal < 0)
            {
                if (maxVal > 0)
                {
                    if (maxVal > Math.Abs(minVal))
                        scalefactor = pGraph.Height / maxVal / 2.2;
                    else
                        scalefactor = pGraph.Height / Math.Abs(minVal) / 2.2;
                }
                else
                    scalefactor = 5.0;
                drawCeroline = true;
            }
            else
            {
                scalefactor = pGraph.Height / (maxVal) / 1.1;
                drawCeroline = false;
            }
            p1 = new Point();
            p2 = new Point();
            Graphics g = pGraph.CreateGraphics();
            if (doClear)
                g.Clear(Color.White);
            Pen bluePen = new Pen(Color.Blue, 2);
            Pen redPen = new Pen(Color.Red, 2);
            Pen blackPen = new Pen(Color.Black, 2);
            if (drawCeroline)
            {
                p1.X = 0;
                p1.Y = (pGraph.Height / 2);
                p2.X = pGraph.Width;
                p2.Y = (pGraph.Height / 2);
                g.DrawLine(blackPen, p1, p2);
            }
            bluePen.Width = 1;
            if (maxTime > 0) // draw interpolated graph
            {
                for (j = 0; j < maxPoints - 1; j++)
                {
                    if ((xp[j] >= minX) && (xp[j] <= maxX))
                    {
                        p1.X = Convert.ToInt32(xp[j] * pGraph.Width / (double)maxTime) + spacing;
                        p2.X = Convert.ToInt32(xp[j + 1] * pGraph.Width / (double)maxTime) + spacing;
                        if (drawCeroline)
                        {
                            p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        else
                        {
                            p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j] * scalefactor)) - spacing;
                            p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor)) - spacing;
                        }
                        g.DrawLine(bluePen, p1, p2);
                    }
                }
                for (j = 0; j < order; j++) // mark reference points
                {
                    p1.X = Convert.ToInt32(x_k[j] * pGraph.Width / (double)maxTime) + spacing;
                    if (drawCeroline)
                        p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor)) - spacing;
                    else
                        p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor)) - spacing;
                    g.DrawEllipse(redPen, (int)(p1.X - 1.0), (int)(p1.Y - 1.0), 2, 2);
                }
            }
            g.Dispose();
        }


        private void button1_MouseClick(object sender, MouseEventArgs e)
        {
            int i;
            DataGridViewCell cell;

            for (i = 0; i < order; i++)
            {
                cell = GDataPoints[i, 0];
                if (cell.Value.ToString() != "")
                {

                    y_k[i] = Convert.ToDouble(cell.Value);
                    cell = GDataPoints[i, 1];
                    x_k[i] = Convert.ToDouble(cell.Value);
                }

                else
                {
                    y_k[i] = 0.0;
                    x_k[i] = 0.0;
                }
            }

            CalcElements(y_k);
            for (i = 0; i < datapoints; i++)
            {
                xp[i] = (double)(i) * x_k[order - 1] / (double)(datapoints);
                yp[i] = Interpolate(xp[i], order);
            }
            DrawGraph(datapoints, x_k[0], x_k[order - 1], true);
        }

        private void pGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(datapoints, x_k[0], x_k[order - 1], false);
        }  
    }
}
